/*
 * Copyright (c) 2016 by ELECTRONIC ASSEMBLY <technik@lcd-module.de>
 * PCT2075 library for arduino.
 *
 * This file is free software; you can redistribute it and/or modify
 * it under the terms of either the GNU General Public License version 2
 * or the GNU Lesser General Public License version 2.1, both as
 * published by the Free Software Foundation.
 */


#include "PCT2075.h"
#include <Wire.h>
#include <Math.h>

#define DAYSEC ((unsigned long)24*3600) 

/*----------------------------
Func: initialize
Desc: Initializes temperature ic PCT2075
Vars: *str = buffer to print strings into
------------------------------*/
void PCT2075::initialize(char *str)
{
  temp_log.ei = 0;
  temp_log.intervall = 1;
  PCT2075::str = str;
}

/*----------------------------
Func: readf
Desc: Returns float temperature
Vars: ---
------------------------------*/
float PCT2075::readf(void)
{
  int value;
  unsigned char val1,val2;
  
  Wire.begin();
  Wire.beginTransmission(PCT_ADR);
  Wire.write(0x00); 
  Wire.endTransmission(true); 
  
  if(2 == Wire.requestFrom(PCT_ADR, 2, true))
  {
    if(Wire.available())
      val1 = Wire.read();
    if(Wire.available())
      val2 = Wire.read();
      
    value = val1<<8;
    value = value + val2;
    value >>= 5;
  }
  return  ((float)value / 8);
}

/*----------------------------
Func: read
Desc: Returns byte temperature. Resolution 0.5 K. Window is between defines LOWTEMP and LOWTEMP+128K
Vars: ---
------------------------------*/
unsigned char PCT2075::read(void)
{
  unsigned char ret;
  float temp = readf();
  if(temp < LOWTEMP)   //make sure that temp is given temp window to store data
    temp = LOWTEMP;
  if(temp > (LOWTEMP+0x7F))  //we need a resolution of 0.5 K -> upper window
    temp = LOWTEMP+0x7F;
    
  temp -= LOWTEMP; //Lowtemp = 0 --> Offset
  ret = (((unsigned char)temp) << 1) + ((unsigned char)temp <= (unsigned char)(temp-0.5));
   
  return ret; 
}

/*----------------------------
Func: printTemp
Desc: Prints actual temperature, Resolution 0.5 K
Vars: ---
------------------------------*/
char *PCT2075::printTemp(void) 
{
  return printTemp(read(), 0);
}

/*----------------------------
Func: printTemp
Desc: Prints user temperature, Resolution 0.5 K
Vars: byte temperature
------------------------------*/
char *PCT2075::printTemp(unsigned char temperature)
{
  return printTemp(temperature, 0);
}

/*----------------------------
Func: printTemp
Desc: Prints temperature into specific location in buffer
Vars: byte temperature, byte index
------------------------------*/
char *PCT2075::printTemp(unsigned char temperature, unsigned char index) 
{
  unsigned char len;
  itoa((temperature >> 1) + LOWTEMP, &str[index], 10);
  len=0;
  while(str[len] != 0)
   len++;

  str[len++] = '.';
  
  if(temperature&1)
    str[len++] = '5';
  else
    str[len++] = '0';
    
  
  str[len++] = 127; //° sign
  str[len++] = 'C';
  str[len] = 0;

  return str;
}

/*----------------------------
Func: printTemp
Desc: Prints actual temperature, Resolution PCT2075 / user
Vars: places
------------------------------*/
char *PCT2075::printTempf(int places) 
{
  unsigned char i;
  float temperature, temp2;
  temperature = readf();
  itoa((int)temperature,str,10);
  i=0;
  while(str[i] != 0)
    i++;

 
  str[i++] = '.';
  
  if(temperature > 0)
   temp2 = temperature - (int)temperature;
  else
   temp2 = (int)temperature - temperature;
  
  itoa(temp2 * pow(10, places), &str[i],10);
  
  i=0;
  while(str[i] != 0)
   i++;
  
  str[i++] = 127; //° sign
  str[i++] = 'C';
  str[i] = 0;

  return str;
}

/*----------------------------
Func: printTempTime
Desc: Prints stored temperature together with timestamp
Vars: places
------------------------------*/
char *PCT2075::printTempTime(unsigned int index) 
{
  unsigned char i = 0;
  unsigned long time_calc;
  if(index >= temp_log.ei)
    index == temp_log.ei - 1;
  
  time_calc = temp_log.start_time + (unsigned long)index*(unsigned long)temp_log.intervall;
  
  if(time_calc > DAYSEC)
    time_calc -= DAYSEC;
  
  str[i++] = (time_calc/36000) + '0';
  str[i++] = (time_calc/3600) % 10 + '0';
  time_calc %= 3600;
  str[i++] = ':';
  str[i++] = (time_calc/600) + '0';
  str[i++] = (time_calc/60) % 10 + '0';
  time_calc %= 60;
  str[i++] = ':';
  str[i++] = (time_calc/10) + '0';
  str[i++] = time_calc % 10 + '0';
  str[i++] = ' ';

  printTemp(temp_log.temp[index], i);

  return str; 
}

/*----------------------------
Func: setupLog
Desc: changes log intervall
Vars: byte intervall (1 = increase 1 step, 2 = decrease 1 step)
------------------------------*/
char* PCT2075::setupLog(unsigned char intervall)
{
  if(intervall == 1 && temp_log.intervall<254)
    temp_log.intervall++;
  else if(intervall == 2 && temp_log.intervall > 1)
    temp_log.intervall--;
  
  itoa(temp_log.intervall/60, str, 10);
  str[1] = ':';
  if(temp_log.intervall%60 < 10)
  {
    str[2] = '0';
    itoa(temp_log.intervall%60, &str[3], 10);
  }
  else
    itoa(temp_log.intervall%60, &str[2], 10);

  str[4] = 0;
  return str;
}

/*----------------------------
Func: startLog
Desc: start temp logging
Vars: time
------------------------------*/
void PCT2075::startLog(unsigned char hour, unsigned char min, unsigned char sec)
{
  temp_log.start_time = (unsigned long)hour*3600 + (unsigned long)min*60 + (unsigned long)sec;
  temp_log.last_time = (temp_log.start_time > temp_log.intervall) ? temp_log.start_time - temp_log.intervall : DAYSEC - temp_log.intervall + temp_log.start_time;
  temp_log.ei = 0;
  storeTemp(hour, min, sec);
}

/*----------------------------
Func: storeTemp
Desc: store temperature
Vars: time
------------------------------*/
void PCT2075::storeTemp(unsigned char hour, unsigned char min, unsigned char sec)
{
  unsigned long time = (unsigned long)hour*3600 + (unsigned long)min*60 + (unsigned long)sec;
  if(time >= temp_log.last_time + temp_log.intervall || time <= temp_log.last_time + temp_log.intervall - DAYSEC)
  {
    temp_log.last_time = time;
    temp_log.temp[temp_log.ei++] = read();
    if(temp_log.ei == MAXPOINTS)
      temp_log.ei = MAXPOINTS-1;
  } 
}

/*----------------------------
Func: getLog
Desc: get logged temps
Vars: *UINT endindex
------------------------------*/
unsigned char *PCT2075::getLog(unsigned int *ei)
{
  *ei = temp_log.ei;

  return temp_log.temp;
}

